import argparse
import fnet.data
import fnet.fnet_model
import json
import logging
import numpy as np
import os
import pdb
import sys
import time
import torch
import warnings

def get_dataloader(remaining_iterations, opts, path_dataset_csv, validation=False):
    transform_signal = [eval(t) for t in opts.transform_signal]
    transform_target = [eval(t) for t in opts.transform_target]
    ds = getattr(fnet.data, opts.class_dataset)(
        path_csv = path_dataset_csv if not validation else opts.path_dataset_val_csv,
        retrain = opts.retrain,
        transform_source = transform_signal,
        transform_target = transform_target,
    )
    ds_patch = fnet.data.BufferedPatchDataset(
        dataset = ds,
        patch_size = opts.patch_size,
        buffer_size = opts.buffer_size if not validation else len(ds),
        buffer_switch_frequency = opts.buffer_switch_frequency if not validation else -1,
        npatches = remaining_iterations*opts.batch_size if not validation else 4*opts.batch_size,
        verbose = True,
        shuffle_images = opts.shuffle_images,
        **opts.bpds_kwargs,
    )
    dataloader = torch.utils.data.DataLoader(
        ds_patch,
        batch_size = opts.batch_size,
    )
    return dataloader

def main():
    parser = argparse.ArgumentParser()
    factor_yx = 0.37241  # 0.108 um/px -> 0.29 um/px
    default_resizer_str = 'fnet.transforms.Resizer((1, {:f}, {:f}))'.format(factor_yx, factor_yx)
    parser.add_argument('--model', type=str, help='ptotein model')
    parser.add_argument('--p', type=float, default=0.05, help='parameter of adjusted loss function')
    parser.add_argument('--batch_size', type=int, default=4, help='size of each batch')
    parser.add_argument('--bpds_kwargs', type=json.loads, default={}, help='kwargs to be passed to BufferedPatchDataset')
    parser.add_argument('--buffer_size', type=int, default=10, help='number of images to cache in memory')
    parser.add_argument('--buffer_switch_frequency', type=int, default=240, help='BufferedPatchDataset buffer switch frequency')
    parser.add_argument('--class_dataset', default='CziDataset', help='Dataset class')
    parser.add_argument('--gpu_ids', type=int, nargs='+', default=0, help='GPU ID')
    parser.add_argument('--interval_save', type=int, default=200, help='iterations between saving log/model')
    parser.add_argument('--iter_checkpoint', nargs='+', type=int, default=1000, help='iterations at which to save checkpoints of model')
    parser.add_argument('--lr', type=float, default=0.0002, help='learning rate')
    parser.add_argument('--n_iter', type=int, default=20000, help='number of training iterations')
    parser.add_argument('--nn_kwargs', type=json.loads, default={}, help='kwargs to be passed to nn ctor')
    parser.add_argument('--nn_module', default='vox2vox', help='name of neural network module: vox2vox or unet')

    parser.add_argument('--loss_coef', default=10, help='constant in loss func')
    parser.add_argument('--loss_func', default='mse', help='loss function choice')
    parser.add_argument('--exclusivity_coef', default=10, help='constant in loss func for exclusivity')
         
    parser.add_argument('--patch_size', nargs='+', type=int, default=[64, 64, 64], help='size of patches to sample from Dataset elements')


    parser.add_argument('--retrain', type=bool, default = False, help='if the model is retrained')

    parser.add_argument('--seed', type=int, default = 29, help='random seed')
    parser.add_argument('--shuffle_images', action='store_true', help='set to shuffle images in BufferedPatchDataset')
    parser.add_argument('--transform_signal', nargs='+', default=['fnet.transforms.normalize'], help='list of transforms on Dataset signal')
    parser.add_argument('--transform_target', nargs='+', default=['fnet.transforms.normalize'], help='list of transforms on Dataset target')
    opts = parser.parse_args()


    if opts.retrain:
        path_run_dir = os.path.join("vox2vox/trainedModels/", opts.model + "_Re")
    else:
        path_run_dir = os.path.join("vox2vox/trainedModels/", opts.model)
    path_dataset_csv = os.path.join("vox2vox/lists/", opts.model + "trainList.csv")
    time_start = time.time()
    if not os.path.exists(path_run_dir):
        os.makedirs(path_run_dir)
    if opts.iter_checkpoint is not None:
        path_checkpoint_dir = os.path.join(path_run_dir, 'checkpoints')
        if not os.path.exists(path_checkpoint_dir):
            os.makedirs(path_checkpoint_dir)

    #Setup logging
    logger = logging.getLogger('model training')
    logger.setLevel(logging.DEBUG)
    fh = logging.FileHandler(os.path.join(path_run_dir, 'run.log'), mode='a')
    sh = logging.StreamHandler(sys.stdout)
    fh.setFormatter(logging.Formatter('%(asctime)s - %(message)s'))
    logger.addHandler(fh)
    logger.addHandler(sh)

    #Set random seed
    if opts.seed is not None:
        np.random.seed(opts.seed)
        torch.manual_seed(opts.seed)
        torch.cuda.manual_seed_all(opts.seed)

    #Instantiate Model
    path_model = os.path.join(path_run_dir, 'model.p')
    if os.path.exists(path_model) :
        print('exist, training with parameter of',opts.p)
        model = fnet.load_model_from_dir(path_run_dir, gpu_ids=opts.gpu_ids)
        logger.info('model loaded from: {:s}'.format(path_model))
    else:
        model = fnet.fnet_model.Model(
            nn_module = opts.nn_module,
            init_weights = True,
            lr = opts.lr,
            model = opts.nn_module,
            loss = opts.loss_func,
            loss_coef = opts.loss_coef,
            exclus_coef = opts.exclusivity_coef,
            retrain = opts.retrain,
            nn_kwargs=opts.nn_kwargs
        )
        logger.info('Model instianted from: {:s}'.format(opts.nn_module))
    logger.info(model)

    #Load saved history if it already exists
    path_losses_csv = os.path.join(path_run_dir, 'losses.csv')
    if os.path.exists(path_losses_csv):
        fnetlogger = fnet.FnetLogger(path_losses_csv)
        logger.info('History loaded from: {:s}'.format(path_losses_csv))
    else:
        fnetlogger = fnet.FnetLogger(columns=['num_iter', 'loss_batch'])

    n_remaining_iterations = max(0, (opts.n_iter - model.count_iter))

    
    dataloader_train = get_dataloader(n_remaining_iterations, opts, path_dataset_csv)

    
    with open(os.path.join(path_run_dir, 'train_options.json'), 'w') as fo:
        json.dump(vars(opts), fo, indent=4, sort_keys=True)

    for i, stacks in enumerate(dataloader_train, model.count_iter):
        if opts.retrain:
            signal, target, m_ = stacks
            loss_batch = model.do_train_iter(signal, target, m_, i)
        else:
            signal, target = stacks
            loss_batch = model.do_train_iter(signal, target, None, i)
        fnetlogger.add({'num_iter': i + 1, 'loss_batch': loss_batch})
        print('{:6d},  {:.3f},  {:.3f},  {:.3f},  {:.3f}, {:.3f}, {:.3f},  {:.3f}'.format(i + 1, loss_batch[0],  loss_batch[1], loss_batch[2][0], loss_batch[3],
        loss_batch[4], loss_batch[5], loss_batch[6]))
        dict_iter = dict(
            num_iter = i + 1,
            loss_batch = loss_batch,
        )
        if ((i + 1) % opts.interval_save == 0) or ((i + 1) == opts.n_iter):
            model.save_state(path_model)
            fnetlogger.to_csv(path_losses_csv)
            logger.info('BufferedPatchDataset buffer history: {}'.format(dataloader_train.dataset.get_buffer_history()))
            logger.info('loss log saved to: {:s}'.format(path_losses_csv))
            logger.info('model saved to: {:s}'.format(path_model))
            
            logger.info('elapsed time: {:.1f} s'.format(time.time() - time_start))
        if (i + 1) % opts.iter_checkpoint == 0:
            path_save_checkpoint = os.path.join(path_checkpoint_dir, 'model_{:06d}.p'.format(i + 1))
            model.save_state(path_save_checkpoint)
            logger.info('model checkpoint saved to: {:s}'.format(path_save_checkpoint))
    

if __name__ == '__main__':
    
    main()
